/* Send a file for flashing */
/* (C)2004 SPROW */

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <string.h>
#include <sys/timeb.h>

#define DEFAULTLPT      1 /* Default to LPT1 */
#define nSTROBE         1
#define BIDIR           32 /* When low, port is output only */
#define ACK             64
#define PORT            (lptport + 0)
#define STATUS          (lptport + 1)
#define CONTROL         (lptport + 2)

/* Function prototypes */
void sendbit(unsigned char);
unsigned long fsize(FILE *);
unsigned int lptport;

int main(int argc, char *argv[])
{
	char            file[256];
	FILE           *in;
	unsigned int    lptnum = 0;
	unsigned char   b;
	unsigned long   ext;
	struct timeb    then, now;

	/* Take a look if the port was overriden */
	if (argc == 2)
	{
		if (stricmp("-lpt1", argv[1])==0) lptnum = 1;
		if (stricmp("-lpt2", argv[1])==0) lptnum = 2;
		if (stricmp("-lpt3", argv[1])==0) lptnum = 3;
	}
	if ((argc > 2) || ((argc == 2)&&(lptnum == 0)))
	{
		printf("MiniB flash uploader.Revision 19-Oct-2008.\n");
		printf("  -lpt<1|2|3> : override default LPT number\n");
		printf("  -help       : show this help\n");
		return 1;
	}
	if (lptnum == 0) lptnum = DEFAULTLPT;

	printf("MiniB flash uploader\n\n");
	/* MiniB will drop ack on exit from the interrupt routine
	 * signalling to sendbit() that it can continue (ie.the
	 * transfer is not interrupt driven at this end) as the lead
	 * is built with BUSY tied low the parallel port cannot be
	 * left to do the transfer itself. */

	/* Read the chosen port base address from the BIOS data area */
	lptport = *((unsigned int far *)MK_FP(0x40, 6 + (2*lptnum)));
	if (lptport == 0)
	{
		printf("Could not open parallel port %d",lptnum);
		return 1;
	}
	outportb(CONTROL, inportb(CONTROL) & ~(BIDIR | nSTROBE)); /* Raise nSTROBE and set DDR to output */

	/* Get file details */
	printf("Enter filename to send: ");
	scanf("%s", &file);
	if ((in = fopen(file, "rb")) == NULL)
	{
		printf("\nCould not open %s", file);
		return 1;
	}

	ftime(&then);
	ext = fsize(in);
	printf("Sent ");
	b = fgetc(in);
	while (!feof(in))
	{
		/* Send out top bit first */
		sendbit(b << 0); sendbit(b << 1); sendbit(b << 2); sendbit(b << 3);
		sendbit(b << 4); sendbit(b << 5); sendbit(b << 6); sendbit(b << 7);
		if (!(ftell(in) % 256))
		{
			gotoxy(6, wherey());
			printf("%d%",(100*ftell(in))/ext);
		}
		b = fgetc(in);
	}

	fclose(in);
	ftime(&now);
	printf("\nDone in %2.2fs\n", (float)((now.time-then.time)+((now.millitm-then.millitm)/1000)));
	/* Expect around 530 bytes per second */

	return 0;
}

void sendbit(unsigned char b)
{
	unsigned long loop;

	outportb(PORT, b); /* Data out */
	outportb(CONTROL, inportb(CONTROL) | nSTROBE); /* Drop nSTROBE */
	for (loop=0; loop<30000; loop++); /* Wait for droopy strobe lines */
	outportb(CONTROL, inportb(CONTROL) & ~nSTROBE); /* Raise nSTROBE */
	while (inportb(STATUS) & ACK); /* Wait for ack */
}

unsigned long fsize(FILE *handle)
{
	unsigned long current, length;

	current = ftell(handle);
	fseek(handle, 0L, SEEK_END);
	length = ftell(handle);
	fseek(handle, current, SEEK_SET);

	return length;
}
